// DirectSound.cpp: CDirectSound NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DirectSound.h"

//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CDirectSound::CDirectSound()
{
	m_pDirectSound = NULL;

	m_bInit = false;
}

CDirectSound::~CDirectSound()
{
	ReleaseAll();
}

void CDirectSound::ReleaseAll()
{
	if( m_pDirectSound!=NULL )
	{
		m_pDirectSound->Release();
		m_pDirectSound=NULL;
	}
	m_bInit = false;
}

bool CDirectSound::Initialize(HWND hwnd)
{
	ReleaseAll();

	HRESULT hr;

	hr = DirectSoundCreate( NULL,&m_pDirectSound,NULL );
	if( hr!=DS_OK )
		return false;

	hr = m_pDirectSound->SetCooperativeLevel( hwnd,DSSCL_NORMAL );
	if( hr!=DS_OK )
		return false;

	m_bInit = true;
	return true;
}

bool CDirectSound::CreateSoundBuffer( CDirectSoundBuffer* pB , LPCTSTR wavName )
{
	if( !m_bInit )
		return false;

	IDirectSoundBuffer* sb;
	PCMWAVEFORMAT pcm;
	DSBUFFERDESC ds;
	HRESULT hr;

	// Open wave file
	FILE* pFile = fopen(wavName,"rb");
	if( !pFile )
		return false;

	// Read in the wave header
	WaveHeader wavHdr;
	fread(&wavHdr, sizeof(wavHdr), 1, pFile);

	// Set up wave format structure.
	memset( &pcm, 0, sizeof(pcm) );
	pcm.wf.wFormatTag = WAVE_FORMAT_PCM;
	// Is this a stereo or mono file?
	pcm.wf.nChannels = (wavHdr.wChnls > 1) ? 2 : 1;
	pcm.wf.nSamplesPerSec = wavHdr.dwSRate;
	pcm.wf.nBlockAlign = (WORD)(wavHdr.wBlkAlign);
	pcm.wf.nAvgBytesPerSec = pcm.wf.nSamplesPerSec * pcm.wf.nBlockAlign;
	pcm.wBitsPerSample = (WORD)(wavHdr.BitsPerSample);

	// Set up DSBUFFERDESC structure.
	memset(&ds, 0, sizeof(ds));
	ds.dwSize = sizeof(ds);
	ds.dwFlags = DSBCAPS_CTRLDEFAULT;
	// Figure out the size of the data region
	ds.dwBufferBytes  = wavHdr.dwDSize;
	ds.lpwfxFormat  = (LPWAVEFORMATEX)&pcm;

	// Create the sound buffer for the wave file
	hr = m_pDirectSound->CreateSoundBuffer(&ds, &sb, NULL);
	if( hr!=DS_OK )
		return false;

	// Seek to correct position in file (if necessary)
	if (sizeof(wavHdr) != 0xffffffff)
		fseek(pFile, sizeof(wavHdr), SEEK_SET);

	// Lock data in buffer for writing
	LPVOID pData1;
	DWORD  dwData1Size;
	LPVOID pData2;
	DWORD  dwData2Size;

	// Read the data for the wave file into the sound buffer
	hr = sb->Lock(0, wavHdr.dwDSize, &pData1, &dwData1Size, &pData2, &dwData2Size, DSBLOCK_FROMWRITECURSOR);
	if( hr!=DS_OK )
		return false;

	// Read in first chunk of data
	if (dwData1Size > 0) 
		fread(pData1, dwData1Size, 1, pFile);

	// read in second chunk if necessary
	if (dwData2Size > 0) 
		fread(pData2, dwData2Size, 1, pFile);

	// Unlock data in buffer
	hr = sb->Unlock(pData1, dwData1Size, pData2, dwData2Size);
	if( hr!=DS_OK )
		return false;

	// Close out the wave file
	fclose(pFile);

	IDirectSoundBuffer* dsb0;
	IDirectSoundBuffer* dsb1;
	IDirectSoundBuffer* dsb2;
	IDirectSoundBuffer* dsb3;
	IDirectSoundBuffer* dsb4;

	m_pDirectSound->DuplicateSoundBuffer(sb,&dsb0);
	m_pDirectSound->DuplicateSoundBuffer(sb,&dsb1);
	m_pDirectSound->DuplicateSoundBuffer(sb,&dsb2);
	m_pDirectSound->DuplicateSoundBuffer(sb,&dsb3);
	m_pDirectSound->DuplicateSoundBuffer(sb,&dsb4);

	pB->Initialize(&sb);
	pB->SetDupBuffer(&dsb0,0);
	pB->SetDupBuffer(&dsb1,1);
	pB->SetDupBuffer(&dsb2,2);
	pB->SetDupBuffer(&dsb3,3);
	pB->SetDupBuffer(&dsb4,4);

	sb->Release();
	dsb0->Release();
	dsb1->Release();
	dsb2->Release();
	dsb3->Release();
	dsb4->Release();

	return true;
}

